/* tslint:disable */
/* eslint-disable */
/**
* Initialize the library
*/
export function init_x12_translate_wasm(): void;
/**
* Formats the given `edi` string into lines, separated by the segment delimiter.
*
* # Exceptions
*
* A call will throw a JS Error IFF it panicked.
* @param {string} x12
* @returns {string}
*/
export function format(x12: string): string;
/**
* Extracts metadata from the given edi file.
*
* # Exceptions
*
* A call will throw a JS Error IFF it panicked.
* @param {string} edi
* @param {MetadataOptions | undefined} options
* @returns {EdiMetadata}
*/
export function metadata(edi: string, options?: MetadataOptions): EdiMetadata;
/**
* Extracts metadata from the given X12 edi file.
*
* # Exceptions
*
* A call will throw a JS Error IFF it panicked.
* @param {string} x12
* @param {MetadataOptions | undefined} options
* @returns {X12Metadata}
*/
export function metadataX12(x12: string, options?: MetadataOptions): X12Metadata;
/**
* Extracts metadata from the given EDIFACT edi file.
*
* # Exceptions
*
* A call will throw a JS Error IFF it panicked.
* @param {string} edifact
* @param {MetadataOptions | undefined} options
* @returns {EdifactMetadata}
*/
export function metadataEdifact(edifact: string, options?: MetadataOptions): EdifactMetadata;
/**
* Generate a 997 acknowledgment for the given edi file.
*
* # Exceptions
*
* A call will throw a JS Error IFF it panicked.
* @param {string} x12
* @param {string} isa_control_number
* @param {string} gs_control_number
* @returns {string | undefined}
*/
export function ack(x12: string, isa_control_number: string, gs_control_number: string): string | undefined;
/**
* Generate a 997 acknowledgment guide json for the given edi file.
*
* # Exceptions
*
* A call will throw a JS Error IFF it panicked.
* @param {string} x12
* @returns {Ack997 | undefined}
*/
export function ackAsJson(x12: string): Ack997 | undefined;
/**
* Generate a 997 acknowledgment edi for the given 997 guide JSON.
*
* # Exceptions
*
* A call will throw a JS Error IFF it panicked.
* @param {Ack997} input
* @param {Envelope} envelope
* @returns {string}
*/
export function generate997(input: Ack997, envelope: Envelope): string;
/**
* Generate an EDI envelope returned as two strings, a header string and a trailer string.
*
* # Exceptions
*
* A call will throw a JS Error IFF it panicked.
* @param {Envelope} envelope
* @param {X12Delimiters | undefined} delimiters
* @returns {EnvelopeParts}
*/
export function generateEnvelope(envelope: Envelope, delimiters?: X12Delimiters): EnvelopeParts;
/**
*/
export enum Standard {
  X12 = 0,
  Edifact = 1,
}
export type EnvelopeParts = [string, string];

export interface AckFunctionalGroupTrailer {
    functional_group_acknowledge_code_01: string;
    number_of_transaction_sets_included_02: number;
    number_of_received_transaction_sets_03: number;
    number_of_accepted_transaction_sets_04: number;
}

export interface AckFunctionalGroupHeader {
    functional_identifier_code_01: string;
    group_control_number_02: number;
}

export interface AckTransactionSetHeader {
    transaction_set_identifier_code_01: string;
    transaction_set_control_number_02: number;
}

export interface AckHeading {
    transaction_set_header_ST: AckTransactionSetHeader;
    functional_group_response_header_AK1: AckFunctionalGroupHeader;
    functional_group_response_trailer_AK9: AckFunctionalGroupTrailer;
}

export interface Ack997 {
    heading: AckHeading;
}

export interface ValidationExceptionField {
    path: string;
    message: string;
}

export interface EdiWarning {
    mark: Mark;
    message: string;
    context?: any;
}

export interface Mark {
    start: Index;
    end?: Index;
}

export interface Index {
    line: number;
    column: number;
}

export interface EdiValidation {
    mark?: Mark;
    message?: string;
    context?: any;
}

export interface EdiError {
    mark: Mark;
    message: string;
    context?: any;
}

export interface Parsed {
    transactionSets?: any[];
    envelope?: ParsedEnvelope;
    delimiters?: X12Delimiters;
}

export interface X12Delimiters {
    element: string;
    composite: string;
    repetition: string;
    segment: string;
}

export interface ParsedEnvelope {
    interchangeHeader?: ParsedInterchangeHeader;
    groupHeader?: ParsedGroupHeader;
    groupTrailer?: ParsedGroupTrailer;
    interchangeTrailer?: ParsedInterchangeTrailer;
}

export interface ParsedInterchangeTrailer {
    numberOfFunctionalGroups?: string;
    controlNumber?: string;
}

export interface InterchangeTrailerMixin {
    numberOfFunctionalGroups?: string;
    controlNumber?: string;
}

export interface ParsedGroupTrailer {
    numberOfTransactions?: string;
    controlNumber?: string;
}

export interface GroupTrailerMixin {
    numberOfTransactions?: string;
    controlNumber?: string;
}

export interface ParsedGroupHeader {
    functionalIdentifierCode?: FunctionalIdentifierCode;
    applicationSenderCode?: string;
    applicationReceiverCode?: string;
    controlNumber?: string;
    date?: string;
    time?: string;
    agencyCode?: AgencyCode;
    release?: string;
}

export type AgencyCode = "T" | "X";

export type FunctionalIdentifierCode = "AA" | "AB" | "AC" | "AD" | "AE" | "AF" | "AG" | "AH" | "AI" | "AK" | "AL" | "AM" | "AN" | "AO" | "AP" | "AR" | "AS" | "AT" | "AU" | "AV" | "AW" | "AX" | "AY" | "AZ" | "BA" | "BB" | "BC" | "BD" | "BE" | "BF" | "BG" | "BL" | "BS" | "CA" | "CB" | "CC" | "CD" | "CE" | "CF" | "CH" | "CI" | "CJ" | "CK" | "CL" | "CM" | "CN" | "CO" | "CP" | "CQ" | "CR" | "CS" | "CT" | "CU" | "CV" | "CW" | "D3" | "D4" | "D5" | "DA" | "DD" | "DF" | "DI" | "DM" | "DS" | "DX" | "EC" | "ED" | "EI" | "EN" | "EO" | "EP" | "ER" | "ES" | "EV" | "EX" | "FA" | "FB" | "FC" | "FG" | "FR" | "FT" | "GC" | "GE" | "GF" | "GL" | "GP" | "GR" | "GT" | "HB" | "HC" | "HI" | "HN" | "HP" | "HR" | "HS" | "HU" | "HV" | "IA" | "IB" | "IC" | "ID" | "IE" | "IF" | "IG" | "IH" | "IJ" | "IM" | "IN" | "IO" | "IR" | "IS" | "JB" | "KM" | "LA" | "LB" | "LI" | "LN" | "LR" | "LS" | "LT" | "MA" | "MC" | "MD" | "ME" | "MF" | "MG" | "MH" | "MI" | "MJ" | "MK" | "MM" | "MN" | "MO" | "MP" | "MQ" | "MR" | "MS" | "MT" | "MV" | "MW" | "MX" | "MY" | "MZ" | "NC" | "NL" | "NP" | "NR" | "NT" | "OC" | "OG" | "OR" | "OW" | "PA" | "PB" | "PC" | "PD" | "PE" | "PF" | "PG" | "PH" | "PI" | "PJ" | "PK" | "PL" | "PN" | "PO" | "PQ" | "PR" | "PS" | "PT" | "PU" | "PV" | "PW" | "PY" | "QG" | "QM" | "QO" | "RA" | "RB" | "RC" | "RD" | "RE" | "RF" | "RG" | "RH" | "RI" | "RJ" | "RK" | "RL" | "RM" | "RN" | "RO" | "RP" | "RQ" | "RR" | "RS" | "RT" | "RU" | "RV" | "RW" | "RX" | "RY" | "RZ" | "SA" | "SB" | "SC" | "SD" | "SE" | "SF" | "SH" | "SI" | "SJ" | "SL" | "SM" | "SN" | "SO" | "SP" | "SQ" | "SR" | "SS" | "ST" | "SU" | "SV" | "SW" | "TA" | "TB" | "TD" | "TE" | "TF" | "TI" | "TJ" | "TM" | "TN" | "TO" | "TP" | "TR" | "TT" | "TU" | "TX" | "UA" | "UB" | "UC" | "UD" | "UI" | "UP" | "UW" | "VA" | "VB" | "VC" | "VD" | "VE" | "VH" | "VI" | "VS" | "WA" | "WB" | "WG" | "WI" | "WL" | "WR" | "WT";

export interface GroupHeaderMixin {
    functionalIdentifierCode?: FunctionalIdentifierCode;
    applicationSenderCode?: string;
    applicationReceiverCode?: string;
    controlNumber?: string;
    date?: string;
    time?: string;
    agencyCode?: AgencyCode;
    release?: string;
}

export interface ParsedInterchangeHeader {
    senderQualifier?: InterchangeIdQualifier;
    senderId?: string;
    receiverQualifier?: InterchangeIdQualifier;
    receiverId?: string;
    controlNumber?: string;
    authorizationInformationQualifier?: AuthorizationInformationQualifier;
    authorizationInformation?: string;
    securityQualifier?: SecurityQualifier;
    securityInformation?: string;
    date?: string;
    time?: string;
    repetitionSeparator?: string;
    acknowledgementRequestedCode?: AcknowledgementRequestedCode;
    usageIndicatorCode?: UsageIndicatorCode;
    componentSeparator?: string;
    controlVersionNumber?: string;
}

export type UsageIndicatorCode = "I" | "P" | "T";

export type AcknowledgementRequestedCode = "0" | "1" | "2" | "3";

export type SecurityQualifier = "00" | "01";

export type AuthorizationInformationQualifier = "00" | "01" | "02" | "03" | "05" | "06" | "07" | "08";

export type InterchangeIdQualifier = "01" | "02" | "03" | "04" | "07" | "08" | "09" | "10" | "11" | "12" | "13" | "14" | "15" | "16" | "17" | "18" | "19" | "20" | "21" | "22" | "23" | "24" | "25" | "26" | "27" | "28" | "29" | "30" | "31" | "32" | "33" | "34" | "35" | "36" | "37" | "38" | "AM" | "NR" | "SA" | "SN" | "ZZ";

export interface InterchangeHeaderMixin {
    senderQualifier?: InterchangeIdQualifier;
    senderId?: string;
    receiverQualifier?: InterchangeIdQualifier;
    receiverId?: string;
    controlNumber?: string;
    authorizationInformationQualifier?: AuthorizationInformationQualifier;
    authorizationInformation?: string;
    securityQualifier?: SecurityQualifier;
    securityInformation?: string;
    date?: string;
    time?: string;
    repetitionSeparator?: string;
    acknowledgementRequestedCode?: AcknowledgementRequestedCode;
    usageIndicatorCode?: UsageIndicatorCode;
    componentSeparator?: string;
    controlVersionNumber?: string;
}

export interface TranslateX12ToJsonOutput {
    output: Parsed;
    warnings?: EdiWarning[];
}

export interface TranslateX12ToJsonInput {
    guideId?: string;
    input: string;
    inputOptions?: TranslateX12ToJsonInputOptions;
    outputOptions?: TranslateX12ToJsonOutputOptions;
}

export interface TranslateX12ToJsonOutputOptions {
    ignoreErrors?: ValidationErrorCode[];
}

export type ValidationErrorCode = "incorrect_length" | "invalid_date" | "invalid_enumeration" | "invalid_number" | "invalid_relational_condition" | "invalid_time" | "missing_required_element" | "pattern_mismatch" | "validations" | "value_out_of_range";

export interface TranslateX12ToJsonInputOptions {
    guidelessInputOptions?: TranslateX12ToJsonGuidelessInputOptions;
}

export interface TranslateX12ToJsonGuidelessInputOptions {
    x12Release?: string;
    delimiters?: X12Delimiters;
}

export interface JsonError {
    code: string;
    path: string[];
    message: string;
}

export interface TranslateJsonToX12Output {
    output?: string;
}

export interface TranslateJsonToX12Input {
    guideId: string;
    input: any;
    envelope?: Envelope;
    outputOptions?: TranslateJsonToX12OutputOptions;
}

export interface TranslateJsonToX12OutputOptions {
    includeTrailingNewlines?: boolean;
}

export interface Envelope {
    interchangeHeader: InterchangeHeader;
    groupHeader: GroupHeader;
    groupTrailer?: GroupTrailer;
    interchangeTrailer?: InterchangeTrailer;
}

export interface InterchangeTrailer {
    numberOfFunctionalGroups: string;
    controlNumber?: string;
}

export interface GroupTrailer {
    numberOfTransactions: string;
    controlNumber?: string;
}

export interface GroupHeader {
    functionalIdentifierCode: FunctionalIdentifierCode;
    applicationSenderCode: string;
    applicationReceiverCode: string;
    controlNumber: string;
    date?: string;
    time?: string;
    agencyCode?: AgencyCode;
    release?: string;
}

export interface InterchangeHeader {
    senderQualifier: InterchangeIdQualifier;
    senderId: string;
    receiverQualifier: InterchangeIdQualifier;
    receiverId: string;
    controlNumber: string;
    authorizationInformationQualifier?: AuthorizationInformationQualifier;
    authorizationInformation?: string;
    securityQualifier?: SecurityQualifier;
    securityInformation?: string;
    date?: string;
    time?: string;
    repetitionSeparator?: string;
    acknowledgementRequestedCode?: AcknowledgementRequestedCode;
    usageIndicatorCode?: UsageIndicatorCode;
    componentSeparator?: string;
    controlVersionNumber?: string;
}

export interface JsonTranslationError {
    errors?: JsonError[];
}

export interface EdiTranslationError {
    partialOutput?: Parsed;
    errors?: EdiError[];
    warnings?: EdiWarning[];
}

export interface BadRequestException {
    message: string;
    code?: string;
}

export interface ValidationException {
    message: string;
    fieldList?: ValidationExceptionField[];
}

export interface SerializationException {
    message: string;
    code?: string;
}

export interface AccessDeniedException {
    message: string;
    code?: string;
}

export interface GatewayTimeoutException {
    message: string;
    code?: string;
}

export interface InternalFailureException {
    message: string;
    code?: string;
}

export interface NotAcceptableException {
    message: string;
    code?: string;
}

export interface NotImplementedException {
    message: string;
    code?: string;
}

export interface QuotaExceededException {
    message: string;
    code?: string;
}

export interface RequestTooLargeException {
    message: string;
    code?: string;
}

export interface ResponseTooLargeException {
    message: string;
    code?: string;
}

export interface ResourceConflictException {
    message: string;
    code?: string;
}

export interface ResourceNotFoundException {
    message: string;
    code?: string;
}

export interface ServiceUnavailableException {
    message: string;
    code?: string;
}

export interface ThrottlingException {
    message: string;
    code?: string;
}

export interface UnknownOperationException {
    message: string;
    code?: string;
}

export interface UnauthorizedException {
    message: string;
    code?: string;
}

export interface UnsupportedMediaTypeException {
    message: string;
    code?: string;
}

export interface EdifactFunctionalGroupSegments {
    ung: string;
    une?: string;
}

export interface EdifactFunctionalGroupEnvelope {
    functionalGroupId: string;
    senderId: string;
    senderIdCodeQualifier?: string;
    recipientId: string;
    recipientIdCodeQualifier?: string;
    date: string;
    time: string;
    functionalGroupReferenceNumber: string;
    controllingAgencyCode: string;
    messageVersionNumber: string;
    messageReleaseNumber: string;
    associationAssignedCode?: string;
    applicationPassword?: string;
    segments: EdifactFunctionalGroupSegments;
}

export interface EdifactFunctionalGroup {
    envelope: EdifactFunctionalGroupEnvelope | null;
    span: Span;
    messages: Message[];
}

export interface Message {
    messageReferenceNumber: string;
    messageType: string;
    messageVersionNumber: string;
    messageReleaseNumber: string;
    controllingAgency: string;
    associationAssignedCode?: string;
    commonAccessReference?: string;
    sequenceOfTransfers?: string;
    firstAndLastTransfer?: string;
    span: Span;
    trackedIdentifiers: Record<string, string[]>;
}

export interface InterchangeSegments {
    unb: string;
    unz?: string;
}

export interface EdifactInterchangeEnvelope {
    syntaxId: string;
    syntaxVersionNumber: string;
    serviceCodeListDirectoryVersionNumber?: string;
    characterEncoding?: string;
    senderId: string;
    senderIdCodeQualifier?: string;
    reverseRoutingAddress?: string;
    interchangeSenderInternalSubId?: string;
    recipientId: string;
    recipientIdCodeQualifier?: string;
    routingAddress?: string;
    interchangeRecipientInternalSubId: string | null;
    date: string;
    time: string;
    interchangeControlReference: string;
    recipientReferencePassword?: string;
    recipientReferencePasswordQualifier?: string;
    applicationReference?: string;
    processingPriorityCode?: string;
    acknowledgmentRequest?: string;
    communicationsAgreementId?: string;
    testIndicator?: string;
    segments: InterchangeSegments;
}

export type Contents = { functionalGroups: EdifactFunctionalGroup[] } | { messages: Message[] } | "empty";

export interface EdifactInterchange {
    envelope: EdifactInterchangeEnvelope;
    standard: Standard;
    span: Span;
    delimiters: Delimiters | null;
    contents: Contents;
}

export interface EdifactMetadata {
    interchanges: EdifactInterchange[];
}

export interface Span {
    start: number;
    end: number;
}

export interface Delimiters {
    segment: string;
    element: string;
    composite: string;
    repetition?: string;
}

export interface X12FunctionalGroupSegments {
    gs: string;
    ge?: string;
}

export interface X12FunctionalGroupEnvelope {
    functionalIdentifierCode: string;
    applicationSenderCode: string;
    applicationReceiverCode: string;
    date: string;
    time: string;
    controlNumber: number;
    responsibleAgencyCode: string;
    release: string;
    segments: X12FunctionalGroupSegments;
}

export interface FunctionalGroup {
    envelope: X12FunctionalGroupEnvelope;
    span: Span;
    transactionSets: TransactionSet[];
}

export interface TransactionSet {
    id: string;
    controlNumber: number;
    span: Span;
    trackedIdentifiers: Record<string, string[]>;
}

export interface InterchangeSegments {
    isa: string;
    iea?: string;
}

export interface X12InterchangeEnvelope {
    senderQualifier: string;
    senderId: string;
    receiverQualifier: string;
    receiverId: string;
    date: string;
    time: string;
    versionNumberCode: string;
    controlNumber: number;
    acknowledgmentRequestedCode: string;
    usageIndicatorCode: string;
    segments: InterchangeSegments;
}

export interface Interchange {
    envelope: X12InterchangeEnvelope;
    standard: Standard;
    span: Span;
    delimiters: Delimiters | null;
    functionalGroups: FunctionalGroup[];
}

export interface X12Metadata {
    interchanges: Interchange[];
}

export type EdiMetadata = ({ standard: "X12" } & X12Metadata) | ({ standard: "Edifact" } & EdifactMetadata);

export interface MetadataOptions {
    identifiersToTrack: string[];
    relaxedEnvelopeValidation: boolean;
}

